@@ -5,9 +5,7 @@
 #include "acceleration/impl/Preconditioner.hpp"
 #include "logging/Logger.hpp"
 
-namespace precice {
-namespace acceleration {
-namespace impl {
+namespace precice::acceleration::impl {
 
 /// Preconditioner that uses the values from the previous time window to scale the quasi-Newton system.
 class ValuePreconditioner : public Preconditioner {
@@ -17,7 +15,7 @@ public:
   /**
    * @brief Destructor, empty.
    */
-  virtual ~ValuePreconditioner() {}
+  ~ValuePreconditioner() override = default;
 
 private:
   logging::Logger _log{"acceleration::ValuePreconditioner"};
@@ -27,11 +25,9 @@ private:
    *
    * @param[in] timeWindowComplete True if this FSI iteration also completed a time window
    */
-  virtual void _update_(bool timeWindowComplete, const Eigen::VectorXd &oldValues, const Eigen::VectorXd &res);
+  void _update_(bool timeWindowComplete, const Eigen::VectorXd &oldValues, const Eigen::VectorXd &res) override;
 
   bool _firstTimeWindow = true;
 };
 
-} // namespace impl
-} // namespace acceleration
-} // namespace precice
+} // namespace precice::acceleration::impl
