@@ -3,22 +3,16 @@
 #include "action/Action.hpp"
 #include "mesh/SharedPointer.hpp"
 
-namespace precice {
-namespace action {
+namespace precice::action {
 
 RecorderAction::RecorderAction(
     Timing               timing,
     const mesh::PtrMesh &mesh)
     : Action(timing, mesh) {}
 
-void RecorderAction::performAction(
-    double time,
-    double timeStepSize,
-    double computedTimeWindowPart,
-    double timeWindowSize)
+void RecorderAction::performAction()
 {
-  records.push_back(Record{
-      getTiming(), time, timeStepSize, computedTimeWindowPart, timeWindowSize});
+  records.push_back(Record{getTiming()});
 }
 
 std::vector<RecorderAction::Record> RecorderAction::records{};
@@ -28,5 +22,4 @@ void RecorderAction::reset()
   records.clear();
 }
 
-} // namespace action
-} // namespace precice
+} // namespace precice::action
