@@ -2,8 +2,7 @@
 
 #include <memory>
 
-namespace precice {
-namespace com {
+namespace precice::com {
 
 class Communication;
 class CommunicationFactory;
@@ -12,5 +11,4 @@ class Request;
 using PtrCommunication        = std::shared_ptr<Communication>;
 using PtrCommunicationFactory = std::shared_ptr<CommunicationFactory>;
 using PtrRequest              = std::shared_ptr<Request>;
-} // namespace com
-} // namespace precice
+} // namespace precice::com
