@@ -7,6 +7,7 @@
 #include <set>
 #include <string>
 #include <vector>
+
 #include "Constants.hpp"
 #include "CouplingData.hpp"
 #include "CouplingScheme.hpp"
@@ -14,12 +15,12 @@
 #include "acceleration/SharedPointer.hpp"
 #include "impl/ConvergenceMeasure.hpp"
 #include "impl/SharedPointer.hpp"
+#include "impl/TimeHandler.hpp"
 #include "io/TXTTableWriter.hpp"
 #include "logging/Logger.hpp"
 #include "m2n/M2N.hpp"
 #include "m2n/SharedPointer.hpp"
 #include "mesh/SharedPointer.hpp"
-#include "utils/assertion.hpp"
 
 namespace precice {
 namespace io {
@@ -51,8 +52,8 @@ class CouplingData;
  * -# query actions and mark them as fulfilled
  * -# compute data to be sent (possibly taking into account received data from
  *    initialize())
- * -# advance the coupling scheme with advance(); where the maximum timestep
- *    length (= time window size) needs to be obeyed
+ * -# advance the coupling scheme with advance(); where the maximum time step
+ *    size (= time window size) needs to be obeyed
  * -# ....
  * -# when the method isCouplingOngoing() returns false, call finalize() to
  *    stop the coupling scheme
@@ -67,45 +68,26 @@ public:
       double                        maxTime,
       int                           maxTimeWindows,
       double                        timeWindowSize,
-      int                           validDigits,
       std::string                   localParticipant,
+      int                           minIterations,
       int                           maxIterations,
       CouplingMode                  cplMode,
-      constants::TimesteppingMethod dtMethod,
-      int                           extrapolationOrder);
-
-  /**
-   * @brief getter for _isInitialized
-   * @returns true, if initialize has been called.
-   */
-  bool isInitialized() const override final
-  {
-    return _isInitialized;
-  }
+      constants::TimesteppingMethod dtMethod);
 
   /**
    * @brief Getter for _sendsInitializedData
    * @returns _sendsInitializedData
    */
-  bool sendsInitializedData() const override final
-  {
-    return _sendsInitializedData;
-  }
+  bool sendsInitializedData() const final override;
 
   /**
-   * @brief Getter for _receivesInitializedData
-   * @returns _receivesInitializedData
+   * @brief getter for _isInitialized
+   * @returns true, if initialize has been called.
    */
-  bool receivesInitializedData() const override final
-  {
-    return _receivesInitializedData;
-  }
+  bool isInitialized() const final override;
 
-  /**
-   * @brief Adds newly computed time. Has to be called before every advance.
-   * @param timeToAdd time to be added
-   */
-  void addComputedTime(double timeToAdd) override final;
+  /// @copydoc cplscheme::CouplingScheme::addComputedTime()
+  bool addComputedTime(double timeToAdd) final override;
 
   /**
    * @brief Returns true, if data will be exchanged when calling advance().
@@ -113,34 +95,30 @@ public:
    * Also returns true after the last call of advance() at the end of the
    * simulation.
    *
-   * @param lastSolverTimestepLength [IN] The length of the last timestep
+   * @param lastSolverTimeStepSize [IN] The size of the last time step
    *        computed by the solver calling willDataBeExchanged().
    */
-  bool willDataBeExchanged(double lastSolverTimestepLength) const override final;
-
-  /**
-   * @brief getter for _hasInitialDataBeenReceived
-   * @returns true, if data has been received in call of initializeData().
-   */
-  bool hasInitialDataBeenReceived() const override final;
+  bool willDataBeExchanged(double lastSolverTimeStepSize) const final override;
 
   /**
    * @brief getter for _hasDataBeenReceived
    * @returns true, if data has been received in last call of advance().
    */
-  bool hasDataBeenReceived() const override final;
+  bool hasDataBeenReceived() const final override;
 
   /**
    * @brief getter for _time
    * @returns the currently computed time of the coupling scheme.
    */
-  double getTime() const override final;
+  double getTime() const final override;
+
+  double getTimeWindowStart() const final override;
 
   /**
    * @brief getter for _timeWindows
    * @returns the number of currently computed time windows of the coupling scheme.
    */
-  int getTimeWindows() const override final;
+  int getTimeWindows() const final override;
 
   /**
    * @brief Function to check whether time window size is defined by coupling scheme.
@@ -151,7 +129,7 @@ public:
    *
    * @returns true, if time window size is available.
    */
-  bool hasTimeWindowSize() const override final;
+  bool hasTimeWindowSize() const final override;
 
   /**
    * @brief Returns the time window size, if one is given by the coupling scheme.
@@ -159,38 +137,33 @@ public:
    * An assertion is thrown, if no valid time window size is given. Check with
    * hasTimeWindowSize().
    */
-  double getTimeWindowSize() const override final;
+  double getTimeWindowSize() const final override;
 
   /**
-   * @brief Returns the remaining timestep length within the current time window.
-   *
-   * If no time window size is prescribed by the coupling scheme, always 0.0 is
-   * returned.
-   */
-  double getThisTimeWindowRemainder() const override final;
-
-  /**
-   * @brief Returns the maximal length of the next timestep to be computed.
+   * @brief Returns the maximal size of the next time step to be computed.
    *
    * If no time window size is prescribed by the coupling scheme, always the
    * maximal double accuracy floating point number value is returned.
    */
-  double getNextTimestepMaxLength() const override final; // @todo mainly used in tests. Is this function actually needed or can we drop it and only use getThisTimeWindowRemainder()?
+  double getNextTimeStepMaxSize() const final override;
 
   /// Returns true, when the coupled simulation is still ongoing.
-  bool isCouplingOngoing() const override final;
+  bool isCouplingOngoing() const final override;
 
   /// Returns true, when the accessor can advance to the next time window.
-  bool isTimeWindowComplete() const override final;
+  bool isTimeWindowComplete() const final override;
 
   /// Returns true, if the given action has to be performed by the accessor.
-  bool isActionRequired(const std::string &actionName) const override final;
+  bool isActionRequired(Action action) const final override;
+
+  /// Returns true, if the given action has to be performed by the accessor.
+  bool isActionFulfilled(Action action) const final override;
 
   /// Tells the coupling scheme that the accessor has performed the given action.
-  void markActionFulfilled(const std::string &actionName) override final;
+  void markActionFulfilled(Action action) final override;
 
   /// Sets an action required to be performed by the accessor.
-  void requireAction(const std::string &actionName) override final;
+  void requireAction(Action action) final override;
 
   /**
    * @brief Returns coupling state information.
@@ -200,36 +173,27 @@ public:
   std::string printCouplingState() const override;
 
   /// Finalizes the coupling scheme.
-  void finalize() override final;
+  void finalize() final override;
 
-  /**
-   * @brief Initializes the coupling scheme.
-   *
-   * @param[in] startTime starting time of coupling scheme
-   * @param[in] startTimeWindow starting counter of time window, from which coupling scheme starts
-   */
-  void initialize(double startTime, int startTimeWindow) override final;
+  /// @copydoc cplscheme::CouplingScheme::initialize()
+  void initialize() final override;
 
-  /**
-   * @brief Initializes data with written values.
-   *
-   * @pre initialize() has been called.
-   * @pre advance() has NOT yet been called.
-   */
-  void initializeData() override final;
+  void reinitialize() final override;
 
-  /**
-   * @brief Advances the coupling scheme.
-   */
-  void advance() override final;
+  ChangedMeshes firstSynchronization(const ChangedMeshes &changes) final override;
+
+  void firstExchange() final override;
+
+  ChangedMeshes secondSynchronization() final override;
+
+  void secondExchange() final override;
 
   /// Adds a measure to determine the convergence of coupling iterations.
   void addConvergenceMeasure(
       int                         dataID,
       bool                        suffices,
       bool                        strict,
-      impl::PtrConvergenceMeasure measure,
-      bool                        doesLogging);
+      impl::PtrConvergenceMeasure measure);
 
   /// Set an acceleration technique.
   void setAcceleration(const acceleration::PtrAcceleration &acceleration);
@@ -238,51 +202,99 @@ public:
    * @brief Getter for _doesFirstStep
    * @returns _doesFirstStep
    */
-  bool doesFirstStep() const
-  {
-    return _doesFirstStep;
-  }
+  bool doesFirstStep() const;
 
   /**
    * @returns true, if coupling scheme has any sendData
    */
   virtual bool hasAnySendData() = 0;
 
-protected:
-  /// Map that links DataID to CouplingData
-  typedef std::map<int, PtrCouplingData> DataMap;
-
-  /// Sends data sendDataIDs given in mapCouplingData with communication.
-  void sendData(const m2n::PtrM2N &m2n, const DataMap &sendData);
-
-  /// Receives data receiveDataIDs given in mapCouplingData with communication.
-  void receiveData(const m2n::PtrM2N &m2n, const DataMap &receiveData);
-
   /**
-   * @brief interface to provide all CouplingData, depending on coupling scheme being used
-   * @return DataMap containing all CouplingData
+   * @brief Determines which data is initialized and therefore has to be exchanged during initialize.
+   *
+   * Calls determineInitialSend and determineInitialReceive for all send and receive data of this coupling scheme.
    */
-  virtual const DataMap getAllData() = 0;
-
-  /**
-   * @brief Function to determine whether coupling scheme is an explicit coupling scheme
-   * @returns true, if coupling scheme is explicit
-   */
-  bool isExplicitCouplingScheme()
-  {
-    PRECICE_ASSERT(_couplingMode != Undefined);
-    return _couplingMode == Explicit;
-  }
+  virtual void determineInitialDataExchange() = 0;
 
   /**
    * @brief Function to determine whether coupling scheme is an implicit coupling scheme
    * @returns true, if coupling scheme is implicit
    */
-  bool isImplicitCouplingScheme()
-  {
-    PRECICE_ASSERT(_couplingMode != Undefined);
-    return _couplingMode == Implicit;
-  }
+  bool isImplicitCouplingScheme() const override;
+
+  /**
+   * @brief Checks if the implicit cplscheme has converged
+   *
+   * @pre \ref doImplicitStep() or \ref receiveConvergence() has been called
+   */
+  bool hasConverged() const override;
+
+protected:
+  /// All send and receive data as a map "data ID -> data"
+  DataMap _allData;
+
+  /// Acceleration method to speedup iteration convergence.
+  acceleration::PtrAcceleration _acceleration;
+
+  void sendTimes(const m2n::PtrM2N &m2n, precice::span<double const> times);
+
+  /**
+   * @brief Sends data sendDataIDs given in mapCouplingData with communication.
+   *
+   * @param m2n M2N used for communication
+   * @param sendData DataMap associated with sent data
+   */
+  void sendData(const m2n::PtrM2N &m2n, const DataMap &sendData);
+
+  std::vector<double> receiveTimes(const m2n::PtrM2N &m2n);
+
+  /**
+   * @brief Receives data receiveDataIDs given in mapCouplingData with communication.
+   *
+   * @param m2n M2N used for communication
+   * @param receiveData DataMap associated with received data
+   */
+  void receiveData(const m2n::PtrM2N &m2n, const DataMap &receiveData);
+
+  /**
+   * @brief Like receiveData, but temporarily sets window time to end of window.
+   *
+   * This function is only needed for SerialCouplingScheme, if substeps="false". Here, a special situation arises for the second participant, because it receives data for the end of the window from the first participant: If substeps="false" only values without timestamps are exchanged. Therefore, getTime() is used to determine the time associated with the values. However, getTime() of the second participant points to the beginning of the window, if we enter a new window. We need to temporarily modify the return value of getTime() to point to the end of the window to be able to store the values at the correct point in time.
+   *
+   * Note: This function could be removed by a) removing the option to turn off exchange of substeps or by b) refactoring the communication such that sent/received values always carry a timestamp.
+   *
+   * @param m2n M2N used for communication
+   * @param receiveData DataMap associated with received data
+   */
+  void receiveDataForWindowEnd(const m2n::PtrM2N &m2n, const DataMap &receiveData);
+
+  /**
+   * @brief Initializes storage in receiveData as zero
+   *
+   * @param receiveData DataMap associated with received data
+   */
+  void initializeWithZeroInitialData(const DataMap &receiveData);
+
+  /**
+   * @brief Adds CouplingData with given properties to this BaseCouplingScheme and returns a pointer to the CouplingData
+   *
+   * If CouplingData with ID of provided data already exists in coupling scheme, no duplicate is created but a pointer to the already existing CouplingData is returned.
+   *
+   * @param data data the CouplingData is associated with
+   * @param mesh mesh the CouplingData is associated with
+   * @param requiresInitialization true, if CouplingData requires initialization
+   * @param exchangeSubsteps true, if CouplingData exchanges all substeps in send/recv
+   * @param direction is the coupling data send or received?
+   *
+   * @return PtrCouplingData pointer to CouplingData owned by the CouplingScheme
+   */
+  PtrCouplingData addCouplingData(const mesh::PtrData &data, mesh::PtrMesh mesh, bool requiresInitialization, bool exchangeSubsteps, CouplingData::Direction direction);
+
+  /**
+   * @brief Function to determine whether coupling scheme is an explicit coupling scheme
+   * @returns true, if coupling scheme is explicit
+   */
+  bool isExplicitCouplingScheme() const;
 
   /**
    * @brief Setter for _timeWindowSize
@@ -291,31 +303,32 @@ protected:
   void setTimeWindowSize(double timeWindowSize);
 
   /**
-   * @brief Getter for _computedTimeWindowPart
-   * @returns _computedTimeWindowPart
+   * @brief Getter for _nextTimeWindowSize
+   * @param timeWindowSize
    */
-  double getComputedTimeWindowPart()
-  {
-    return _computedTimeWindowPart;
-  }
+  double getNextTimeWindowSize() const;
+
+  /**
+   * @brief Setter for _nextTimeWindowSize
+   * @param timeWindowSize
+   */
+  void setNextTimeWindowSize(double timeWindowSize);
 
   /**
    * @brief Setter for _doesFirstStep
    */
-  void setDoesFirstStep(bool doesFirstStep)
-  {
-    _doesFirstStep = doesFirstStep;
-  }
-
-  /**
-   * @brief Used to set flag after initialData has been received. Automatically calls checkDataHasBeenReceived().
-   */
-  void checkInitialDataHasBeenReceived();
+  void setDoesFirstStep(bool doesFirstStep);
 
   /**
    * @brief Used to set flag after data has been received using receiveData().
    */
-  void checkDataHasBeenReceived();
+  void notifyDataHasBeenReceived();
+
+  /**
+   * @brief Getter for _receivesInitializedData
+   * @returns _receivesInitializedData
+   */
+  bool receivesInitializedData() const;
 
   /**
    * @brief Setter for _timeWindows
@@ -325,42 +338,28 @@ protected:
    *
    * @param timeWindows number of time windows
    */
-  void setTimeWindows(int timeWindows)
-  {
-    _timeWindows = timeWindows;
-  }
-
-  /**
-   * @brief Reserves memory to store data values from previous iterations and time windows in coupling data and acceleration, and initializes with zero.
-   */
-  void initializeStorages();
+  void setTimeWindows(int timeWindows);
 
   /**
    * @brief sends convergence to other participant via m2n
    * @param m2n used for sending
-   * @param convergence bool that is being sent
    */
-  void sendConvergence(const m2n::PtrM2N &m2n, bool convergence);
+  void sendConvergence(const m2n::PtrM2N &m2n);
 
   /**
    * @brief receives convergence from other participant via m2n
    * @param m2n used for receiving
    * @returns convergence bool
    */
-  bool receiveConvergence(const m2n::PtrM2N &m2n);
+  void receiveConvergence(const m2n::PtrM2N &m2n);
 
   /**
    * @brief perform a coupling iteration
-   * @returns whether this iteration has converged or not
+   * @see hasConverged
    *
    * This function is called from the child classes
    */
-  bool doImplicitStep();
-
-  /**
-   * @brief stores current data in buffer for extrapolation
-   */
-  void storeExtrapolationData();
+  void doImplicitStep();
 
   /**
    * @brief finalizes this window's data and initializes data for next window.
@@ -370,13 +369,7 @@ protected:
   /**
    * @brief used for storing all Data at end of doImplicitStep for later reference.
    */
-  void storeIteration()
-  {
-    PRECICE_ASSERT(isImplicitCouplingScheme());
-    for (const DataMap::value_type &pair : getAllData()) {
-      pair.second->storeIteration();
-    }
-  }
+  void storeIteration();
 
   /**
    * @brief Sets _sendsInitializedData, if sendData requires initialization
@@ -391,9 +384,30 @@ protected:
   void determineInitialReceive(DataMap &receiveData);
 
   /**
-   * @brief getter for _extrapolationOrder
+   * @brief Function to check whether end of time window is reached. Does not check for convergence
+   * @returns true if end time of time window is reached or if this participant defines time window size (participant first method)
    */
-  int getExtrapolationOrder();
+  bool reachedEndOfTimeWindow() const;
+
+  /// @copydoc cplscheme::CouplingScheme::requiresSubsteps()
+  bool requiresSubsteps() const final override;
+
+  /// @copydoc cplscheme::CouplingScheme::implicitDataToReceive()
+  ImplicitData implicitDataToReceive() const override;
+
+  /// @copydoc cplscheme::CouplingScheme::localParticipant()
+  std::string localParticipant() const final override;
+
+protected:
+  /**
+   * @return the end of the time window, defined as timeWindowStart + timeWindowSize
+   */
+  double getWindowEndTime() const;
+
+  /**
+   * @return the start of the time window
+   */
+  double getWindowStartTime() const;
 
 private:
   /// Coupling mode used by coupling scheme.
@@ -401,12 +415,9 @@ private:
 
   mutable logging::Logger _log{"cplscheme::BaseCouplingScheme"};
 
-  /// Maximum time being computed. End of simulation is reached, if _time == _maxTime
+  /// Maximum time being computed. End of simulation is reached, if getTime() == _maxTime
   double _maxTime;
 
-  /// current time; _time <= _maxTime
-  double _time = 0;
-
   /// Number of time windows that have to be computed. End of simulation is reached, if _timeWindows == _maxTimeWindows
   int _maxTimeWindows;
 
@@ -414,10 +425,16 @@ private:
   int _timeWindows = 0;
 
   /// size of time window; _timeWindowSize <= _maxTime
-  double _timeWindowSize;
+  double _timeWindowSize = UNDEFINED_TIME_WINDOW_SIZE;
 
-  /// Part of the window that is already computed; _computedTimeWindowPart <= _timeWindowSize
-  double _computedTimeWindowPart = 0;
+  /// time window size of next window (acts as buffer for time windows size provided by first participant, if using first participant method)
+  double _nextTimeWindowSize = UNDEFINED_TIME_WINDOW_SIZE;
+
+  /// Time handler
+  impl::TimeHandler _time;
+
+  /// Lower limit of iterations during one time window. Prevents convergence if _iterations < _minIterations.
+  int _minIterations = -1;
 
   /// Limit of iterations during one time window. Continue to next time window, if _iterations == _maxIterations.
   int _maxIterations = -1;
@@ -431,31 +448,27 @@ private:
   /// True, if local participant is the one starting the explicit scheme.
   bool _doesFirstStep = false;
 
-  /// True, if _computedTimeWindowPart == _timeWindowSize and (coupling has converged or _iterations == _maxIterations)
+  /// True, if _time == _timeWindowStartTime + _timeWindowSize and (coupling has converged or _iterations == _maxIterations)
   bool _isTimeWindowComplete = false;
 
-  /// Acceleration method to speedup iteration convergence.
-  acceleration::PtrAcceleration _acceleration;
-
   /// True, if this participant has to send initialized data.
   bool _sendsInitializedData = false;
 
   /// True, if this participant has to receive initialized data.
   bool _receivesInitializedData = false;
 
-  /// True, if initialData has been received from other participant. Flag is used to make sure that coupling scheme is implemented and used correctly.
-  bool _hasInitialDataBeenReceived = false;
-
   /// True, if data has been received from other participant. Flag is used to make sure that coupling scheme is implemented and used correctly.
   bool _hasDataBeenReceived = false;
 
   /// True, if coupling has been initialized.
   bool _isInitialized = false;
 
-  /// True, if initialize data has been called.
-  bool _initializeDataHasBeenCalled = false;
+  std::set<Action> _requiredActions;
 
-  std::set<std::string> _actions;
+  std::set<Action> _fulfilledActions;
+
+  /// True if implicit scheme converged
+  bool _hasConverged = false;
 
   /// Responsible for monitoring iteration count over time window.
   std::shared_ptr<io::TXTTableWriter> _iterationsWriter;
@@ -466,25 +479,6 @@ private:
   /// Local participant name.
   std::string _localParticipant = "unknown";
 
-  /**
-   * Order of predictor of interface values for first participant.
-   *
-   * The first participant in the implicit coupling scheme has to take some
-   * initial guess for the interface values computed by the second participant.
-   * In order to improve this initial guess, an extrapolation from previous
-   * time windows can be performed.
-   *
-   * The standard predictor is of order zero, i.e., simply the converged values
-   * of the last time windows are taken as initial guess for the coupling iterations.
-   * Currently, an order 1 predictor (linear extrapolation) and order 2 predictor
-   * (see https://doi.org/10.1016/j.compstruc.2008.11.013, p.796, Algorithm line 1 )
-   * is implement besides that.
-   */
-  const int _extrapolationOrder;
-
-  /// Smallest number, taking validDigits into account: eps = std::pow(10.0, -1 * validDigits)
-  const double _eps;
-
   /**
    * @brief Holds meta information to perform a convergence measurement.
    * @param data Associated data field
@@ -499,11 +493,10 @@ private:
     bool                        suffices;
     bool                        strict;
     impl::PtrConvergenceMeasure measure;
-    bool                        doesLogging;
 
     std::string logHeader() const
     {
-      return "Res" + measure->getAbbreviation() + "(" + couplingData->getDataName() + ")";
+      return fmt::format("Res{}({}:{})", measure->getAbbreviation(), couplingData->getMeshName(), couplingData->getDataName());
     }
   };
 
@@ -518,30 +511,28 @@ private:
   /// Functions needed for initialize()
 
   /**
-   * @brief implements functionality for initialize in base class.
+   * @brief Need to initialize receive data
    */
-  virtual void initializeImplementation() = 0;
-
-  /// Functions needed for initializeData()
+  virtual void initializeReceiveDataStorage() = 0;
 
   /**
-   * @brief implements functionality for initializeData in base class.
+   * @brief implements functionality for initialize in base class.
    */
   virtual void exchangeInitialData() = 0;
 
   /// Functions needed for advance()
 
-  /**
-   * @brief implements functionality for advance in base class.
-   * @returns true, if iteration converged
-   */
-  virtual bool exchangeDataAndAccelerate() = 0;
+  /// Exchanges the first set of data
+  virtual void exchangeFirstData() = 0;
+
+  /// Exchanges the second set of data
+  virtual void exchangeSecondData() = 0;
 
   /**
    * @brief interface to provide accelerated data, depending on coupling scheme being used
    * @return data being accelerated
    */
-  virtual const DataMap getAccelerationData() = 0;
+  virtual DataMap &getAccelerationData() = 0;
 
   /**
    * @brief If any required actions are open, an error message is issued.
@@ -549,10 +540,9 @@ private:
   void checkCompletenessRequiredActions();
 
   /**
-   * @brief Function to check whether end of time window is reached. Does not check for convergence
-   * @returns true if end time of time window is reached.
+   * @brief Issues an error if coupling data does not contain stamples.
    */
-  bool reachedEndOfTimeWindow();
+  void checkCouplingDataAvailable();
 
   /**
    * @brief Initialize txt writers for iterations and convergence tracking
