@@ -2,9 +2,7 @@
 #include "logging/LogMacros.hpp"
 #include "math/differences.hpp"
 
-namespace precice {
-namespace cplscheme {
-namespace impl {
+namespace precice::cplscheme::impl {
 
 ResidualRelativeConvergenceMeasure::ResidualRelativeConvergenceMeasure(double convergenceLimitPercent)
     : _convergenceLimitPercent(convergenceLimitPercent)
@@ -12,6 +10,4 @@ ResidualRelativeConvergenceMeasure::ResidualRelativeConvergenceMeasure(double co
   PRECICE_ASSERT(math::greater(_convergenceLimitPercent, 0.0) && math::greaterEquals(1.0, _convergenceLimitPercent),
                  "Relative convergence limit has to be in ]0;1] !");
 }
-} // namespace impl
-} // namespace cplscheme
-} // namespace precice
+} // namespace precice::cplscheme::impl
