@@ -1,13 +1,12 @@
-#include "Tracer.hpp"
 #include <string>
 #include <utility>
-#include "logging/Logger.hpp"
 
-namespace precice {
-namespace logging {
+#include "logging/Tracer.hpp"
+
+namespace precice::logging {
 
 Tracer::Tracer(
-    Logger &    log,
+    Logger     &log,
     LogLocation loc)
     : _log(log),
       _loc(std::move(loc))
@@ -19,5 +18,4 @@ Tracer::~Tracer()
   _log.trace(_loc, std::string{"Leaving "}.append(_loc.func));
 }
 
-} // namespace logging
-} // namespace precice
+} // namespace precice::logging
