@@ -5,8 +5,7 @@
 #include "m2n/DistributedCommunication.hpp"
 #include "mesh/SharedPointer.hpp"
 
-namespace precice {
-namespace m2n {
+namespace precice::m2n {
 
 class PointToPointComFactory : public DistributedComFactory {
 
@@ -14,12 +13,11 @@ public:
   explicit PointToPointComFactory(com::PtrCommunicationFactory comFactory);
 
   DistributedCommunication::SharedPointer newDistributedCommunication(
-      mesh::PtrMesh mesh);
+      mesh::PtrMesh mesh) override;
 
 private:
   /// communication factory for 1:M communications
   com::PtrCommunicationFactory _comFactory;
 };
 
-} // namespace m2n
-} // namespace precice
+} // namespace precice::m2n
