@@ -14,11 +14,9 @@
 #include "testing/Testing.hpp"
 #include "utils/assertion.hpp"
 
-namespace precice {
-namespace mesh {
+namespace precice::mesh {
 class Vertex;
-} // namespace mesh
-} // namespace precice
+} // namespace precice::mesh
 
 using namespace precice;
 using precice::testing::TestContext;
@@ -39,7 +37,7 @@ std::vector<double> readDoublesFromTXTFile(std::string const &filename, int skip
 }
 } // namespace
 
-void testWatchPoint(const TestContext & context,
+void testWatchPoint(const TestContext  &context,
                     bool                withEdge,
                     std::vector<double> watchPosition,
                     std::vector<double> expected)
@@ -75,38 +73,20 @@ void testWatchPoint(const TestContext & context,
   }
 
   using precice::testing::operator""_dataID;
-  PtrData                 doubleData   = mesh->createData("DoubleData", 1, 0_dataID);
-  PtrData                 vectorData   = mesh->createData("VectorData", 2, 1_dataID);
-  auto &                  doubleValues = doubleData->values();
-  auto &                  vectorValues = vectorData->values();
-  mesh->allocateDataValues();
+  PtrData                 doubleData = mesh->createData("DoubleData", 1, 0_dataID);
+  PtrData                 vectorData = mesh->createData("VectorData", 2, 1_dataID);
 
   if (context.size > 1) {
     if (context.isPrimary()) {
-      doubleValues(0) = 1.0;
-      doubleValues(1) = 2.0;
-      vectorValues(0) = 1.0;
-      vectorValues(1) = 2.0;
-      vectorValues(2) = 3.0;
-      vectorValues(3) = 4.0;
+      doubleData->setSampleAtTime(0, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{1.0, 2.0}}));
+      vectorData->setSampleAtTime(0, time::Sample(vectorData->getDimensions(), Eigen::VectorXd{{1.0, 2.0, 3.0, 4.0}}));
     } else {
-      doubleValues(0) = 2.0;
-      doubleValues(1) = 3.0;
-      vectorValues(0) = 3.0;
-      vectorValues(1) = 4.0;
-      vectorValues(2) = 5.0;
-      vectorValues(3) = 6.0;
+      doubleData->setSampleAtTime(0, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{2.0, 3.0}}));
+      vectorData->setSampleAtTime(0, time::Sample(vectorData->getDimensions(), Eigen::VectorXd{{3.0, 4.0, 5.0, 6.0}}));
     }
   } else {
-    doubleValues(0) = 1.0;
-    doubleValues(1) = 2.0;
-    doubleValues(2) = 3.0;
-    vectorValues(0) = 1.0;
-    vectorValues(1) = 2.0;
-    vectorValues(2) = 3.0;
-    vectorValues(3) = 4.0;
-    vectorValues(4) = 5.0;
-    vectorValues(5) = 6.0;
+    doubleData->setSampleAtTime(0, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{1., 2., 3.}}));
+    vectorData->setSampleAtTime(0, time::Sample(vectorData->getDimensions(), Eigen::VectorXd{{1., 2., 3., 4., 5., 6.}}));
   }
 
   std::string filename0("precice-WatchPointTest-timeseries-0.log");
@@ -127,35 +107,23 @@ void testWatchPoint(const TestContext & context,
     // Change data (next timestep)
     if (context.size > 1) {
       if (context.isPrimary()) {
-        doubleValues(0) = 2.0;
-        doubleValues(1) = 3.0;
-        vectorValues(0) = 2.0;
-        vectorValues(1) = 3.0;
-        vectorValues(2) = 4.0;
-        vectorValues(3) = 5.0;
+        doubleData->setSampleAtTime(1, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{2., 3.}}));
+        vectorData->setSampleAtTime(1, time::Sample(vectorData->getDimensions(), Eigen::VectorXd{{2., 3., 4., 5.}}));
       } else {
-        doubleValues(0) = 3.0;
-        doubleValues(1) = 4.0;
-        vectorValues(0) = 4.0;
-        vectorValues(1) = 5.0;
-        vectorValues(2) = 6.0;
-        vectorValues(3) = 7.0;
+        doubleData->setSampleAtTime(1, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{3., 4.}}));
+        vectorData->setSampleAtTime(1, time::Sample(vectorData->getDimensions(), Eigen::VectorXd{{4., 5., 6., 7.}}));
       }
     } else {
-      doubleValues(0) = 2.0;
-      doubleValues(1) = 3.0;
-      doubleValues(2) = 4.0;
-      vectorValues(0) = 2.0;
-      vectorValues(1) = 3.0;
-      vectorValues(2) = 4.0;
-      vectorValues(3) = 5.0;
-      vectorValues(4) = 6.0;
-      vectorValues(5) = 7.0;
+      doubleData->setSampleAtTime(1, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{2., 3., 4.}}));
+      vectorData->setSampleAtTime(1, time::Sample(vectorData->getDimensions(), Eigen::VectorXd{{2., 3., 4., 5., 6., 7.}}));
     }
 
     // Write output again
     watchpoint0.exportPointData(1.0);
 
+    doubleData->moveToNextWindow();
+    vectorData->moveToNextWindow();
+
     // Write output again to check if the data stays the same
     watchpoint0.exportPointData(2.0);
   }
@@ -179,9 +147,10 @@ void testWatchPoint(const TestContext & context,
 
 BOOST_AUTO_TEST_SUITE(WatchPoint)
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(TimeSeriesNoEdgeSerialPoint1)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   bool withEdge           = false;
   auto watchPointPosition = std::vector<double>{-0.5, 0.6};
   auto expected           = std::vector<double>{
@@ -191,9 +160,10 @@ BOOST_AUTO_TEST_CASE(TimeSeriesNoEdgeSerialPoint1)
   testWatchPoint(context, withEdge, watchPointPosition, expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(TimeSeriesWithEdgeSerialPoint1)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   bool withEdge           = true;
   auto watchPointPosition = std::vector<double>{-0.5, 0.6};
   auto expected           = std::vector<double>{
@@ -203,9 +173,10 @@ BOOST_AUTO_TEST_CASE(TimeSeriesWithEdgeSerialPoint1)
   testWatchPoint(context, withEdge, watchPointPosition, expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(TimeSeriesNoEdgeSerialPoint2)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   bool withEdge           = false;
   auto watchPointPosition = std::vector<double>{0.0, 1.6};
   auto expected           = std::vector<double>{
@@ -215,9 +186,10 @@ BOOST_AUTO_TEST_CASE(TimeSeriesNoEdgeSerialPoint2)
   testWatchPoint(context, withEdge, watchPointPosition, expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(TimeSeriesWithEdgeSerialPoint2)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   bool withEdge           = true;
   auto watchPointPosition = std::vector<double>{0.0, 1.6};
   auto expected           = std::vector<double>{
@@ -227,9 +199,10 @@ BOOST_AUTO_TEST_CASE(TimeSeriesWithEdgeSerialPoint2)
   testWatchPoint(context, withEdge, watchPointPosition, expected);
 }
 
+PRECICE_TEST_SETUP(""_on(2_ranks).setupIntraComm(), Require::Events)
 BOOST_AUTO_TEST_CASE(TimeSeriesNoEdgeParallelPoint1)
 {
-  PRECICE_TEST(""_on(2_ranks).setupIntraComm(), Require::Events);
+  PRECICE_TEST();
   bool withEdge           = false;
   auto watchPointPosition = std::vector<double>{-0.5, 0.6};
   auto expected           = std::vector<double>{
@@ -239,9 +212,10 @@ BOOST_AUTO_TEST_CASE(TimeSeriesNoEdgeParallelPoint1)
   testWatchPoint(context, withEdge, watchPointPosition, expected);
 }
 
+PRECICE_TEST_SETUP(""_on(2_ranks).setupIntraComm(), Require::Events)
 BOOST_AUTO_TEST_CASE(TimeSeriesWithEdgeParallelPoint1)
 {
-  PRECICE_TEST(""_on(2_ranks).setupIntraComm(), Require::Events);
+  PRECICE_TEST();
   bool withEdge           = true;
   auto watchPointPosition = std::vector<double>{-0.5, 0.6};
   auto expected           = std::vector<double>{
@@ -251,9 +225,10 @@ BOOST_AUTO_TEST_CASE(TimeSeriesWithEdgeParallelPoint1)
   testWatchPoint(context, withEdge, watchPointPosition, expected);
 }
 
+PRECICE_TEST_SETUP(""_on(2_ranks).setupIntraComm(), Require::Events)
 BOOST_AUTO_TEST_CASE(TimeSeriesNoEdgeParallelPoint2)
 {
-  PRECICE_TEST(""_on(2_ranks).setupIntraComm(), Require::Events);
+  PRECICE_TEST();
   bool withEdge           = false;
   auto watchPointPosition = std::vector<double>{0.0, 1.6};
   auto expected           = std::vector<double>{
@@ -263,9 +238,10 @@ BOOST_AUTO_TEST_CASE(TimeSeriesNoEdgeParallelPoint2)
   testWatchPoint(context, withEdge, watchPointPosition, expected);
 }
 
+PRECICE_TEST_SETUP(""_on(2_ranks).setupIntraComm(), Require::Events)
 BOOST_AUTO_TEST_CASE(TimeSeriesWithEdgeParallelPoint2)
 {
-  PRECICE_TEST(""_on(2_ranks).setupIntraComm(), Require::Events);
+  PRECICE_TEST();
   bool withEdge           = true;
   auto watchPointPosition = std::vector<double>{0.0, 1.6};
   auto expected           = std::vector<double>{
@@ -275,9 +251,10 @@ BOOST_AUTO_TEST_CASE(TimeSeriesWithEdgeParallelPoint2)
   testWatchPoint(context, withEdge, watchPointPosition, expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(Reinitialize)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   using namespace mesh;
   using Eigen::VectorXd;
   // Setup geometry
@@ -289,19 +266,13 @@ BOOST_AUTO_TEST_CASE(Reinitialize)
   mesh::Vertex &v3 = mesh->createVertex(Eigen::Vector2d(1.0, 2.0));
   mesh->createEdge(v1, v2);
 
-  PtrData doubleData   = mesh->createData("DoubleData", 1, 0_dataID);
-  PtrData vectorData   = mesh->createData("VectorData", 2, 1_dataID);
-  auto &  doubleValues = doubleData->values();
-  auto &  vectorValues = vectorData->values();
-  mesh->allocateDataValues();
+  PtrData doubleData = mesh->createData("DoubleData", 1, 0_dataID);
+  PtrData vectorData = mesh->createData("VectorData", 2, 1_dataID);
 
   // v1, v2 carry data 1
   // v2 and later v3 carry data 2
-  doubleValues.setConstant(1.0);
-  doubleValues(2) = 2.0;
-  vectorValues.setConstant(1.0);
-  vectorValues(4) = 2.0;
-  vectorValues(5) = 2.0;
+  doubleData->setSampleAtTime(0, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{1., 1., 2.}}));
+  vectorData->setSampleAtTime(0, time::Sample(vectorData->getDimensions(), Eigen::VectorXd{{1., 1., 1., 1., 2., 2.}}));
 
   std::string filename0("precice-WatchPointTest-reinit-0.log");
   std::string filename1("precice-WatchPointTest-reinit-1.log");
@@ -326,15 +297,8 @@ BOOST_AUTO_TEST_CASE(Reinitialize)
     mesh::Vertex &v4 = mesh->createVertex(Eigen::Vector2d(1.0, 0.0));
     mesh->createEdge(v3, v4);
     mesh->index().clear();
-    mesh->allocateDataValues();
-    doubleValues.setConstant(1.0);
-    doubleValues(2) = 2.0;
-    doubleValues(3) = 2.0;
-    vectorValues.setConstant(1.0);
-    vectorValues(4) = 2.0;
-    vectorValues(5) = 2.0;
-    vectorValues(6) = 2.0;
-    vectorValues(7) = 2.0;
+    doubleData->setSampleAtTime(0, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{1., 1., 2., 2.}}));
+    vectorData->setSampleAtTime(0, time::Sample(vectorData->getDimensions(), Eigen::VectorXd{{1., 1., 1., 1., 2., 2., 2., 2.}}));
 
     // Re-Initialize
     watchpoint0.initialize();
@@ -379,9 +343,10 @@ BOOST_AUTO_TEST_CASE(Reinitialize)
   }
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(VolumetricInterpolation2D)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   using namespace mesh;
   using Eigen::VectorXd;
   // Setup geometry
@@ -396,18 +361,12 @@ BOOST_AUTO_TEST_CASE(VolumetricInterpolation2D)
   mesh->createEdge(v3, v1);
   mesh->createTriangle(v1, v2, v3);
 
-  PtrData doubleData   = mesh->createData("DoubleData", 1, 0_dataID);
-  PtrData vectorData   = mesh->createData("VectorData", 2, 1_dataID);
-  auto &  doubleValues = doubleData->values();
-  auto &  vectorValues = vectorData->values();
-  mesh->allocateDataValues();
+  PtrData doubleData = mesh->createData("DoubleData", 1, 0_dataID);
+  PtrData vectorData = mesh->createData("VectorData", 2, 1_dataID);
 
   // Data is (1,1,2) for the scalar, and same for each vector component.
-  doubleValues.setConstant(1.0);
-  doubleValues(2) = 2.0;
-  vectorValues.setConstant(1.0);
-  vectorValues(4) = 2.0;
-  vectorValues(5) = 2.0;
+  doubleData->setSampleAtTime(0, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{1., 1., 2., 2.}}));
+  vectorData->setSampleAtTime(0, time::Sample(vectorData->getDimensions(), Eigen::VectorXd{{1., 1., 1., 1., 2., 2., 2., 2.}}));
 
   std::string filename0("precice-WatchPointTest-volumetric2d-0.log");
   std::string filename1("precice-WatchPointTest-volumetric2d-1.log");
@@ -461,9 +420,10 @@ BOOST_AUTO_TEST_CASE(VolumetricInterpolation2D)
   }
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(VolumetricInterpolation3D)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   using namespace mesh;
   using Eigen::VectorXd;
   // Setup geometry
@@ -477,15 +437,10 @@ BOOST_AUTO_TEST_CASE(VolumetricInterpolation3D)
 
   mesh->createTetrahedron(v1, v2, v3, v4);
 
-  PtrData doubleData   = mesh->createData("DoubleData", 1, 0_dataID);
-  auto &  doubleValues = doubleData->values();
-  mesh->allocateDataValues();
+  PtrData doubleData = mesh->createData("DoubleData", 1, 0_dataID);
 
   // Data is (1,1,2) for the scalar, and same for each vector component.
-  doubleValues(0) = 1.0;
-  doubleValues(1) = 2.0;
-  doubleValues(2) = 3.0;
-  doubleValues(3) = 4.0;
+  doubleData->setSampleAtTime(0, time::Sample(doubleData->getDimensions(), Eigen::VectorXd{{1., 2., 3., 4.}}));
 
   std::string filename0("precice-WatchPointTest-volumetric3d-0.log");
   std::string filename1("precice-WatchPointTest-volumetric3d-1.log");
@@ -539,9 +494,10 @@ BOOST_AUTO_TEST_CASE(VolumetricInterpolation3D)
   }
 }
 
+PRECICE_TEST_SETUP(""_on(4_ranks).setupIntraComm(), Require::Events)
 BOOST_AUTO_TEST_CASE(VolumetricParallel)
 {
-  PRECICE_TEST(""_on(4_ranks).setupIntraComm(), Require::Events);
+  PRECICE_TEST();
   using namespace mesh;
   using Eigen::VectorXd;
   // Setup geometry
@@ -574,14 +530,10 @@ BOOST_AUTO_TEST_CASE(VolumetricParallel)
   } break;
   }
 
-  PtrData doubleData   = mesh->createData("DoubleData", 1, 0_dataID);
-  auto &  doubleValues = doubleData->values();
-  mesh->allocateDataValues();
+  PtrData doubleData = mesh->createData("DoubleData", 1, 0_dataID);
 
   // Data is (1, 2, 3, 4) on the tetra, other ranks agree on their subset
-  for (int i = 0; i <= context.rank; ++i) {
-    doubleValues(i) = i + 1;
-  }
+  doubleData->setSampleAtTime(0, time::Sample(doubleData->getDimensions(), Eigen::VectorXd(context.rank + 1).setLinSpaced(1., context.rank + 1)));
 
   std::string filename0("precice-WatchPointTest-volumetricParallel-0.log");
   bool        isClosest;
